/*************************************************************************
  program name: lm_lods.c (lange-gamma-version -- renamed lm_markers)
             mow modified for quantitative traits -- file renamed lm_quant

  Program to do lod scores (replaces homozyg.c from old Autozyg)
  The state (X) consists of segregation indicators -- so can generalize to
        cases of more than one observed/pedigree, and multiple loci.

  The data, Y, is the multilocus genotypes of observed individuals
     in the pedigree.

  Obtained by modification of jnt_auto.c.  December 1998.  (EAT)

Here is the plan!
    OK this is the PLAN  (12/15/98)

OK, now we have a general autozyg program, we need to go back to
lod estimation, for those as want.  We use the same program (modified).

Since the data are locus-by-locus, the likelihood ratio contribution is
just ratio of P_theta(S)/P_sim_th(S).

This is very easy, once we get the chromosome set-ups fixed.

1) Trait locus data set-up -- assume trait-locus genotypes for observed
individuals -- done  12/20/98
2) Likelihood-ratio contributions routine,  -- done 12/18/98
3) Simulation and evaluation chromosomes for each location. -- done 12/23/98

Now to integrate the L-sampler, using the new Lsampler and Msampler
      libraries and new routines;        EAT March 2000

Converted to MORGAN_V2.3                 MAJ June 2000

Modifications to specify simulation points, within each interval and
outside of map.    EAT, then converted to MORGAN 2.3 by MAJ, Nov 2000.

Modification of full_seg_prob, and recombinant counts     EAT  Nov 2000
    scoring of counts and tot_log_probs by component and locus/interval.

Modifications to lik_rat, do_lsmpl, etc.                  EAT  Dec 2000

Modifications to use specify intervals for l-ratio updating and use
    more efficient seg_prob updating routines for l-sample steps
                                                          EAT Jan 2001

Added changes for sequential imputation set-up            AWG March 2001

But now we can use locus_mjrpeel with sample_flag zero
     to do a Lange-style lod score using only sampled S_M    EAT May 2001
  We can use the extval and intval positions as computation positions
    -- except we can also compute at the loci, maybe??

This is the version to test new trait stuctures.
Program renamed lm_markers.                                 EAT March 2002

Version modified for quantitative traits (lm_quant.c)       EAT Jan 2004

**************************************************************************/

/**
 * @defgroup lm_markers lm_markers
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_markers"
#define NH  80                          /* reinstated, EAT, for test positions  */

#define QUANT 0

#include "autozyg.h"
#include "quant.h"

#undef CHECK

void  chkprnt1 (int);
void  chkprnt2a (int);
void  chkprnt2b (int);
void  chkprnt5 (void);
void  chkprnt6 (void);
void  chkprnt7 (int);
void  chkprnt8 (int);
void  chkprnt9 (int);
void  chkprnt10 (int);
void  chkprnt11 (int, int, int, int);
void  chkprnt17 (void);

int   main (int argc, char *argv[])
{
   int   i, j, k;
   int **dodo = NULL;
   double **quant = 0;
   int   t, c, h, err;
   int   ngenes = 0, nfou, nloc, ntp, ntraits, nmarkers;
   int  *loc_flags;
   /* flags marker loci specified as traits */
   int  *nns, nsw, sx, nh;
   int  *perm_sw, *perm_loc = 0;
   /* permutation arrays of switches and loci */
   int ***y;                            /* full genotypic array */

   int   bsz, bc;                       /* btch-size = NIt/20,
                                           bc = current batch                 */
   double *rb_prob, **rb_prbm;          /* cumulative rb_likelihood
                                           and batch totals position nh       */
   double *rb_sd;                       /* Monte Carlo SDevs                  */
/* WERE:  rb_prob[NH], rb_prbm[20][NH], rb_sd[NH]                             */

   double base_lik;                     /* base-point P(Y_T)                  */
   /* Variables added for a quantitative trait */
   double ve = 0.0;                     /* Residual variance */
   /* double va = 0.0; *//* Additive variance */
   double mu = 0.0;                     /* Global mean       */
   /* double *additive = NULL; *//* Additive genetic component,
      updated when va > 0 */
/* double *ym = NULL;                      Centered trait values */
/* double lnlik;                                                 */

   int ***recounts;
   double **prob;
   /* likelihoods etc.        */
   double **tot_log_prob_seg;
   double **tot_log_prob_pen;
   double **temp_log_prob_seg;
   int   tot_count;

   int   jj;
   double rec0[3] =
   {.5, .5, .5};
   int   loc0[3] =
   {0, 0, 0};

   /* double cumupos[2]; *//* Map position (cM) of current
      test position */
   double markpos[2];                   /* Map position (cM) of current
                                           marker */
   int   lastmarker = 0;                /* Last test position */

   double **rec_left;                   /* fem =0, male=1; by test position   */
   double **rec_rght;                   /* fem =0, male=1; by test position   */
   int  *loc_left;                      /* locus left of test position        */
   int  *loc_rght;                      /* locus rght of test position        */
/* WERE:  double rec_left[2][NH], double rec_rght[2][NH],                     */
/*        int    loc_left[NH],    int    loc_rght[NH]                         */

   FILE *pedF, *oSeedF;

   atexit (print_tail);

   print_head (*argv);

   if (!proc_pars (argc, argv, PNAM))   /* process input parameter      */
      exit (EXIT_FAILURE);              /* statements                   */

   if (!proc_auto ())                   /* check statements for         */
      exit (EXIT_FAILURE);              /* completeness and consistency */
   /*                                    * and setup program controls   */

   if ((QUANT) && (!proc_quant ()))     /* set up quant trait           */
      exit (EXIT_FAILURE);

   if (!proc_seeds ())                  /* set up seed                  */
      exit (EXIT_FAILURE);              /* parameters                   */

   if (!proc_fils ())                   /* set up pedigree file         */
      exit (EXIT_FAILURE);              /* parameters                   */

   setSeeds (SAMPL);                    /* initializes seeds for rans() */

   if (!chk_pedfil ())                  /* checks pedigree file for     */
      give_up (ERR_qit);                /* errors.                      */

   if (!mak_opedfil ())                 /* writes new pedigree file     */
      give_up (ERR_qit);                /* if changes                   */

   if (!(pedF = open_ped ()))           /*opens pedigree file           */
      give_up (ERR_qit);

   alloc_ped ();
   if (QUANT)
      input_data (pedF, 0, 0);          /*input pedigree and trait data */
   else
      input_ped (pedF, 0, dodo, quant);

   nloc = NLoci - 1;                    /*nloc is the number of markers */

   nfou = count_founders ();

   MaxNsw = 2 * (pedsize - nfou);
   nns = malloc ((n_compnts + 1) * sizeof (int));
   alloc_chk (WHERE, nns);

   loc_flags = malloc ((nloc + 1) * sizeof (int));
   alloc_chk (WHERE, loc_flags);

/* set up trait structures */
/* read (1) or set (0) information about the Traits (including markers)   */

   ntraits = set_traits (NamdLoc, nloc, loc_flags, pedF);
   nmarkers = print_traits (ntraits, loc_flags);

/* now allocate space for genotypes y for our values of pedsize, nloc   */

   y = alloc_y (nmarkers, pedsize);

/* For a quantitative phenotypic trait, the following is done by input_data */

   if ((TraitPheno) && (!QUANT))        /* phenotypic discrete trait     */
   {
      qdex = calloc ((pedsize + 1), sizeof (int));
      alloc_chk (WHERE, qdex);
      trtphen = calloc ((pedsize + 1), sizeof (int));
      alloc_chk (WHERE, qdex);
   }

/* now set marker data for observed individuals  */

   set_markdat (y, nloc, n_compnts);

/* now set trait data                            */

   if (!QUANT)
   {
      set_traitdat (y, qdex, trtphen, pedsize, n_compnts);
      prnt_traitdat (y, pedsize, n_compnts);
   }
   else
   {
      for (i = 0; i < pedsize; ++i)
      {
         printf ("\n  %3d %s  %d", i, nghd[i].self, qdex[i]);
         if (qdex[i] > 0)
            printf (" %f ", x[0][qdex[i]]);
      }
      printf ("\n");
   }

   chkprnt1 (nloc);

/* allocate arrays xx0, xx1, gg0, gg1 for meiosis indicators and genes */

   alloc_xxgg (nloc, pedsize);

/* now set and index switches  */

   nsw = list_switches (nloc, nns);
   printf (" Number of meiosis switches to sample is %d\n", nsw);
   fflush (stdout);

/* allocate and set arrays of marker and trait positions and recombination
   fractions  (for sampler setup, trait is unlinked) */

   alloc_chromo (nloc, 0);

   for (sx = FEM; sx <= MAL; sx++)
      set_chromosomes (nloc, Marek[sx], 0, loc0, rec0, sx);

/* set up permutation arrays for loci and switches */
   if (Scan)
   {
      perm_sw = calloc (nsw, sizeof (int));
      alloc_chk (WHERE, perm_sw);
      for (i = 0; i < nsw; ++i)
         perm_sw[i] = i;
      perm_loc = calloc (nloc + 1, sizeof (int));
      alloc_chk (WHERE, perm_loc);
      for (i = 0; i <= nloc; ++i)
         perm_loc[i] = i;
   }

   /* Allocate locus structures    -moved to main program, EAT June 2003 */
   /* pen_flag=4, gives qtlpen for quant trait  */

   if (QUANT)
      allocate_loci (nloc + 1, NAll, Qq, pedF, 4);
   else
      allocate_loci (nloc + 1, NAll, Qq, pedF, 1);

   fclose (pedF);

   /* routine set_quant_trait should replace schl_set_traitdat for schnell */
   if (QUANT)
   {                                    /* set quant trait locus model                */
      mu = set_quant_trait (nloc);
      /* allocate and set ym                        */
      alloc_quantdat (np);
      for (j = 1; j <= np; j++)
         ym[j] = x[0][j] - mu;

      set_seg_mode (0);                 /* ?? what does this do ?? */
   }

/* set up initial xx0, xx1 (meiosis indicators) */
   if (SmplSetup == LSMPL)              /* use locus-by-locus sampling */
   {
      if (!QUANT)
         set_lsmpl (0, nloc, y, TraitPheno);
      else
         set_lsmpl (0, nloc, y, 4);
   }
   else
      /* use sequential imputation */ if (!QUANT)
      seq_imp (NSeqImp, pedsize, nloc + 1, y, (double *) 0, 1.0);
   else
      seq_imp (NSeqImp, pedsize, nloc + 1, y, ym, ve);

   peel_trace = TRACE_LEVEL_0;          /* Turn on trace if level >0 */

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);

/* Added EAT -- P(Y_T) */
/*   set_transmission (loci, 0, nloc);   */
   set_simple_trans (loci, -1, -1, 0.5, 0.5, 0.5, 0.5);
   if (!QUANT)
      base_lik = locus_mjrpeel (0, 1.0, 1.0, 0, loci);
   else
      base_lik = locus_mjrpeel (ym, loci[0].ve, 1.0, 0, loci);

   /*   checks for Zero probability in peeling operation                      */
   if (base_lik == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   printf (" Base likelihood P(Y_T) = %e\n \n", exp (base_lik));

   k = 0;
   for (j = 0; j <= nloc; ++j)
   {
      ngenes = set_genes (j);

      chkprnt2a (j);

      /* Allocate memory for gen_pen() */

      if (loci[j].locus_type == TRAIT_LOCUS)
         alloc_gen_pen (ngenes, j);
      else if (!k++)
         alloc_seg_pen (ngenes, nloc + 1, y, &prob);
   }

   for (j = 0; j <= nloc; ++j)
   {
      for (c = 0; c < n_compnts; ++c)
         prob[c][j] = seg_pen2 (c, y, Qq[j], j, ngenes, &err);

      chkprnt2b (j);
   }

   /* now set evaluation points for lange_lik */
   /* ntp is number of trait positions plus number of markers (NLocPos) + 1  */

   alloc_lf_rt (NLocPos + 1, &rec_left, &rec_rght, &loc_left, &loc_rght);

   ntp = set_eval_points (nloc, loc_left, loc_rght,
                          rec_left[0], rec_rght[0], rec_left[1], rec_rght[1]);

   if (ntp != NLocPos + 1)
      give_up_where (WHERE, ERA_mrk, NLocPos, ntp);

   alloc_rb (ntp, 20, &rb_prob, &rb_prbm, &rb_sd);

   for (nh = 0; nh < ntp; ++nh)
   {
      rb_prob[nh] = 0.0;
      rb_sd[nh] = 0.0;
      for (jj = 0; jj < 20; ++jj)
         rb_prbm[jj][nh] = 0.0;
   }

   if ((NIt / ScorIt) * ScorIt != NIt)
      printf ("\n WARNING Scoring interval does not divide realizations count");
   tot_count = NIt / ScorIt;
   if ((tot_count / 20) * 20 != tot_count)
      printf ("\n WARNING Batch number does not divide scored iterations");
   bsz = tot_count / 20;

   printf (" %d realizations will be scored, at scoring interval %d\n",
           tot_count, ScorIt);
   printf (" Scored realizations will be batched in 20 sets of size %d", bsz);
   printf ("\n \n");
   fflush (stdout);

   for (c = 0; c < n_compnts; ++c)
   {
      for (h = 0; h <= nloc; ++h)
      {
         tot_log_prob_seg[c][h] = 0.0;
         tot_log_prob_pen[c][h] = 0.0;
      }
      for (j = 0; j <= nloc; ++j)
      {
         for (jj = 0; jj < 6; ++jj)
            recounts[c][j][jj] = 0;
      }
   }

   tot_count = 0;                       /* reset tot_count to count when
                                           scored */
   bc = 0;                              /* current batch indicator */

   fflush (stdout);
   chkprnt9 (k);

   alloc_logprob ();

   alloc_msmpl ();

   prnt_mcmcstrt ();

   /* now do burn-in iterations */

   printf (" Now starting %d iterations of burn-in\n", NBrnIt);
   for (t = 1; t <= NBrnIt; t++)
   {
      if (!(t % ChkIt))
      {
         printf (" Starting burn-in iteration %d\n", t);
         fflush (stdout);
      }
      sampling (t, 1, nloc, nsw, ngenes, perm_loc, perm_sw, y, prob, QUANT);
   }

   /* now do MCMC sampling, and score results */

   for (t = 0; t < NIt; ++t)
   {
      chkprnt10 (nloc);
      sampling (t, 1, nloc, nsw, ngenes, perm_loc, perm_sw, y, prob, QUANT);
      /* MCMC sampling step or scan */

      if (!((t + 1) % ScorIt))
      {                                 /* score tot log-probs and recombs */
         for (c = 0; c < n_compnts; ++c)
         {
            full_seg_prob (nloc, c, recounts, temp_log_prob_seg);
            for (j = 0; j < nloc; ++j)
            {
               tot_log_prob_pen[c][j] += log (prob[c][j]);
               tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];
            }
            tot_log_prob_pen[c][nloc] += log (prob[c][nloc]);
         }

         /* score rb_likelihoods  */
/* zero out meiosis indicators for line-by-line check
   for (i=0; i<pedsize; ++i)
   { if (nghd[i].pa  != &nullnode)
   { for (j=0; j<=nloc; ++j)
   { xx0[j][i] =0; xx1[j][i] =0; } } }
 */
/* start new batch if current btch is complete */

         do_rblik (ntp, loc_left, loc_rght, rec_left[1], rec_left[0],
                   rec_rght[1], rec_rght[0], rb_prob, rb_prbm[bc],
                   base_lik, QUANT);
         ++tot_count;

         if (!((tot_count) % bsz))
            ++bc;
      }
   }
   printf ("\n");
   prnt_mcmcend ();

/* output results for this simulation.estimation set */

   for (c = 0; c < n_compnts; ++c)
   {
      pr_cmpnthead (c);

      pr_log_prob_seg (1, nloc, -nns[c + 1], tot_log_prob_seg[c], tot_count);

      pr_log_prob_pen (1, nloc, tot_log_prob_pen[c], tot_count);

      pr_reco_counts (1, nloc, recounts[c]);
   }

/* print out likelihood estimates  */

   for (k = 1; k < ntp; ++k)
      if (loc_left[k] != loc_rght[k])
      {
         rb_prob[k] = rb_prob[k] / tot_count;
         for (bc = 0; bc < 20; ++bc)
            rb_prbm[bc][k] = rb_prbm[bc][k] / bsz;
      }

/* Batch-means standard devation estimates  */

   for (k = 1; k < ntp; ++k)
   {
      if (loc_left[k] != loc_rght[k])
      {
         for (bc = 0; bc < 20; ++bc)
            rb_sd[k] += (rb_prbm[bc][k] - rb_prob[k])
               * (rb_prbm[bc][k] - rb_prob[k]);
         rb_sd[k] = sqrt (rb_sd[k] / (19 * 20));
      }
   }

   prntLodSHead ();

   for (k = 1; k < ntp; ++k)
   {
      if (!prntLodSPos (k - 1))
         /* the current test position is a marker position */
         printf (" %11s%10s\n", "NA", "NA");
      else
         printf (" %11.4f%10.4f\n", log10 (rb_prob[k]),
                 rb_sd[k] / (rb_prob[k] * log (10.0)));
   }
   printf ("\n");
   fflush (stdout);

/* now we are done with MCMC */

   fflush (stdout);

   /* Free up allocated memory */
   free_xxgg ();
   free_allefrq ();
   free_chromo (0);
   free (loc_flags);
   free_y (&y);
   free_allocate_loci ();

   locus_peel_free ();

   chkprnt17 ();

   /* now we are done  */

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   exit (EXIT_SUCCESS);

}

/****************************************************************/

void  chkprnt1 (int nloc)
{
#ifdef CHECK
   int   i, j;
/* Check out all the data -- trait genotypes and markers   */
   for (i = 0; i < pedsize; ++i)
   {
      printf (" ind = %d", nghd[i].self);
      for (j = 0; j <= nloc; ++j)
         printf ("    %d %d", y[0][j][i], y[1][j][i]);
      printf ("\n");
   }
   printf ("\n");
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt2a (int j)
{
#ifdef CHECK
   printf (" Set genes at locus %d\n", j);
   fflush (stdout);
   print_obs_genes (j, y);
   fflush (stdout);
   printf (" Scored prob at locus %d; giving", j);
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt2b (int j)
{
#ifdef CHECK
   int   c;
   for (c = 0; c < n_compnts; ++c)
      printf (" %7.4f", prob[c][j]);
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt5 (void)
{
#ifdef CHECK
   printf (" Now to call set_positions\n\n");
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt6 (void)
{
#ifdef CHECK
   printf (" Now did call set_lsampler\n\n");
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt7 (int j)
{
#ifdef CHECK
   printf (" Reset trait locus meioses, position %d\n\n", j);
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt8 (int k)
{
#ifdef CHECK
   printf (" Done set-up %d \n\n", k);
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt9 (int k)
{
#ifdef CHECK
   printf (" Initial seg lhood-ratios for simulation chromosome %d\n\n", k);
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt10 (int nloc)
{                                       /* print set-up if needed for chk */
#ifdef CHECK
   int   j;
   for (j = 0; j <= nloc; ++j)
   {
      printf (" Locus %d position %d set-up \n", j, chromo[0][j]);
      print_set_up (j);
      printf ("\n");
      fflush (stdout);
   }
   printf ("\n");
   fflush (stdout);
#endif /*CHECK */
   return;
}
void  chkprnt11 (int l, int i, int sx, int c)
{
#ifdef CHECK
   printf (" sw %d, ind %d %d, cmp %d\n\n",
           l, nghd[i].self, sx, c);
   fflush (stdout);
#endif /*CHECK */
   return;
}

void  chkprnt17 (void)
{
#ifdef CHECK
   printf (" \n Now we are done freeing\n\n ");
   fflush (stdout);
#endif /*CHECK */
   return;
}

/** @} */
