/*
 * Write and retrieve checkpoints information for lm_lods
 * $Id: lods_chk.c,v 1.2 2003/06/09 18:07:41 nali Exp $
 * */

static char *orescName[2];                  /* name of the output rescue files */

static void lods_dump (double ***lratios, int traitpos)
{
   FILE *fdump;
   unsigned int cseed;
   unsigned int tseed;
   int   c, i, j;

   static int whichdump = -1;               /* This variable alternates
                                             * between 0 and 1, indicating
                                             * which dump file to use */
   /* skip if no rescue file is specified */
   if (get_filent (ORESC)->fname != NULL)
   {
      /* First call sets up the filenames */
      if (whichdump == -1)
      {
         for (i = 0; i < 2; ++i)
         {
            orescName[i] = malloc (strlen (get_filent (ORESC)->fname) + 5);
            strcpy (orescName[i], get_filent (ORESC)->fname);
            strncat (orescName[i], i == 0 ? "1" : "2", 1);
         }
         whichdump = 0;
      }

      /* open rescue file, ready to be written */
      fdump = fopen (orescName[whichdump], "w");
      if (fdump == NULL)
      {
         fprintf (stderr, "Failed to open rescue file %s.\n",
                  orescName[whichdump]);
         exit (EXIT_FAILURE);
      }

      /* random seeds */
      getsd (&cseed, &tseed);
      fprintf (fdump, " %08x %08x\n", cseed, tseed);
      fflush (fdump);

      /* xx0, xx1 */
      for (j = 0; j <= pedsize; ++j)
      {
         for (i = 0; i < NLoci; ++i)
            fprintf (fdump, "%d %d\n", xx0[i][j], xx1[i][j]);
         fflush (fdump);
      }

      /* no. of trait positons finished */
      fprintf (fdump, "%d\n", traitpos);

      /* left and right sims */
      for (i = 0; i <= traitpos; ++i)
         for (j = 0; j < NTPos; ++j)
         {
            for (c = 0; c < n_compnts; ++c)
               fprintf (fdump, "%f ", lratios[c][i][j]);
            fprintf (fdump, "\n");
         }

      /* finished dummping */
      fprintf (fdump, "%d CHECKPOINTS_OK\n", traitpos);
      fclose (fdump);
      whichdump = 1 - whichdump;
   }
}

static int lods_restart (double ***lratios, int *ptraitpos)
{
   /* check point */
   FILE *frcvr = NULL;                      /* input rescue file handle */
   unsigned int cseed;
   unsigned int tseed;
   int   c, i, j;

   /* Try to access the rescue files */
   if (get_filent (IRESC)->fname != NULL)
   {
      frcvr = open_fil (IRESC);
      if (frcvr != NULL)
      {
         fprintf (stderr, "Opened input rescue file %s.\n",
                  get_filent (IRESC)->fname);

         /* Process input rescue file ... */
         /* recover seeds */
         fscanf (frcvr, "%x %x", &cseed, &tseed);
         setsd (cseed, tseed);

         /* xx0, xx1 */
         for (j = 0; j <= pedsize; ++j)
            for (i = 0; i < NLoci; ++i)
               fscanf (frcvr, "%d %d", &xx0[i][j], &xx1[i][j]);
         for (i = 0; i < NLoci; ++i)
            set_genes (i);

         /* no. of positions finished */
         fscanf (frcvr, "%d", ptraitpos);
         ++*ptraitpos;
         printf ("Restart from trait position %d.\n", *ptraitpos);

         for (i = 0; i <= *ptraitpos; ++i)
            for (j = 0; j < NTPos; ++j)
               for (c = 0; c < n_compnts; ++c)
                  fscanf (frcvr, "%lf", &lratios[c][i][j]);

         fclose (frcvr);
         return 1;
      }
      else
      {
         fprintf (stderr, "Failed to open input rescue file %s.\n",
                  get_filent (IRESC)->fname);
         give_up (ERR_qit);
         return 2;
      }
   }
   return 0;
}

static void finish_dumping ()
{
   if (orescName[0])
      free (orescName[0]);
   if (orescName[1])
      free (orescName[1]);
}
