/*************************************************************************
  program name: make_data.c   (EAT, 1998-9)
  
  Program to simulate segregations conditional on a partially specified
     trait-locus segregation pattern, check patterns and allocate alleles

  Modified from jnt_auto.c, to use same input structures and routines

  Originally written to generate the icemod data -- see book section 10.1

 Updated from MORGAN_F1 to MORGAN 2.3.2  (EAT, Jan 2002)
    temporarily needs own  read_gen_dat3, trait_segs3 and next_marker_segs3
      as these are gone from MORGAN 2.3.2 and/or have not been updated
      in Msample/realize.c

 Updated to Morgan 2.4 (EAT, Feb 2002)
    get rid of extra routines. Use lm_auto statements.

**************************************************************************/

/**
 * @defgroup markerdrop markerdrop
 * @ingroup GDrop
 * @{
 */

#define PNAM  "markerdrop"
#define IBD
#define MDP
#include "genedrop.h"                   /* MORGAN 2.5 version  EAT */

#define DIE {fprintf(stderr, "main prog: calloc failed\nExiting\n"); exit(1);}

/* int *trtphen, *qdex; */

int   main (int argc, char *argv[])

{
   int   i, ii, j, k, sx;
   int **dodo = 0;
   double **quant = 0;
   int   ngenes = 0, nfou, nloc, nint, nt, nmarkers, ntraits;
   int   nsw, *nns;
   int  *loc_flags;

   int ***y;

   int **xxx = 0;

   double gendist (), recomb ();
   FILE *pedF;
   FILE *oSeedF;

   atexit (print_tail);

   print_head (*argv);

   if (!proc_pars (argc, argv, PNAM))   /* process input parameter      */
      exit (EXIT_FAILURE);              /* statements                   */

   if (!proc_auto ())                   /* check statements for         */
      exit (EXIT_FAILURE);              /* completeness and consistency */
   /* and prepare controls for     */
   /* lm_lods                      */
   NColData = 8;
   for (j = 0; j < 5; ++j)
   {
      SelInt[j] = j + 1;
   }
   SelInt[5] = 6;                       /* No material change -- just   */
   SelInt[6] = 7;                       /* making explicit that current */
   /* partial trait segs are currently in cols 5+1 and 6+1, and will go */
   /* into data array items 6 and 7    */

   if (!proc_seeds ())                  /* set up seed                  */
      exit (EXIT_FAILURE);              /* parameters                   */

   if (!proc_fils ())                   /* set up pedigree file         */
      exit (EXIT_FAILURE);              /* parameters                   */

   setSeeds (SAMPL);                    /* initializes seeds for rans() */

   if (!chk_pedfil ())                  /* checks pedigree file for     */
      give_up (ERR_qit);                /* errors.                      */

   if (!mak_opedfil ())                 /* writes new pedigree file     */
      give_up (ERR_qit);                /* if changes                   */

   if (!(pedF = open_ped ()))           /*opens pedigree file           */
      give_up (ERR_qit);

   alloc_ped ();
   input_ped (pedF, 0, dodo, quant);    /*input pedigree and trait data */
   fclose (pedF);

   nloc = NLoci - 1;

   nfou = count_founders ();
   printf ("\n # founders = %d", nfou);
   printf ("\n # components = %d", n_compnts);

/* now set and index switches  */

   alloc_xxgg (nloc, pedsize);

   MaxNsw = 2 * (pedsize - nfou);
   nns = malloc ((n_compnts + 1) * sizeof (int));
   alloc_chk (WHERE, nns);

   nsw = list_switches (nloc, nns);

   printf ("\n have now set switches");
   fflush (stdout);

   loc_flags = malloc ((nloc + 1) * sizeof (int));
   alloc_chk (WHERE, loc_flags);

/* allocate and set arrays of marker and trait positions  and 
   recombination fractions  */

   alloc_chromo (nloc, 0);

   for (sx = FEM; sx <= MAL; sx++)
      set_chromosomes (nloc, Marek[sx], 0, &Loc0, &Rec0[sx], sx);

/* set up trait structures */
/* read (1) or set (0) information about the Traits (including markers)   */

   ntraits = set_traits (NamdLoc, nloc, loc_flags, pedF);
   nmarkers = print_traits (ntraits, loc_flags);

/* now allocate space for genotypes y for our values of pedsize, nloc   */

   y = alloc_y (nloc, pedsize);

   qdex = malloc ((pedsize + 1) * sizeof (int));
   alloc_chk (WHERE, qdex);
   for (i = 0; i <= pedsize; ++i)
      qdex[i] = 0;
   trtphen = malloc ((pedsize + 1) * sizeof (int));
   alloc_chk (WHERE, trtphen);
   for (i = 0; i <= pedsize; ++i)
      trtphen[i] = 0;

/* now set marker data for observed individuals  */

#if 0
   if (nmarkers < nloc)
   {
      set_markmap (nloc, nmarkers, loc_flags);
      nloc = nmarkers;
      NLoci = nloc + 1;
   }
#endif

   set_markdat (y, nloc, n_compnts);

   printf ("\n check input marker data");

   for (i = 0; i < NObs; ++i)
   {
      k = ObsIx[i];
      printf ("\n %s", nghd[k].self);
      for (j = 0; j < nloc; ++j)
         printf (" %d %d", y[0][j + 1][k], y[1][j + 1][k]);
   }

/* now set trait segregations from input values */

   trait_segs (nsw);
/*
   printf ("\n check %d switches and trait indicators", nsw);  fflush(stdout);
   for (i=1; i<= nsw; ++i) {  j=(meiosis[i].ind).index; 
   if ( meiosis[i].sex ==0) xxx=xx0; else xxx=xx1;
   printf("\n sw %d ind %d %d %s sx %d xxx %d",i,j,
   nghd[j].index, nghd[j].self, meiosis[i].sex, xxx[0][j]); }
 */
   fflush (stdout);

   nt = revchro[0][0];                  /* position of trait */
   printf (" \n trait interval %d recom to next marker malrec= %f femrec = %f",
           nt, xecomo[1][0][nt], xecomo[0][0][nt]);
   fflush (stdout);

/* set other markers to the right  */
   if (nt < nloc)
   {
      next_marker_segs (nsw, 0, nt + 1, nt);
      for (j = nt + 1; j < nloc; ++j)
         next_marker_segs (nsw, j, j + 1, j);
   }

/* set other markers to the left */
   if (nt > 0)
   {
      next_marker_segs (nsw, 0, nt, nt - 1);
      for (j = nt; j > 1; --j)
         next_marker_segs (nsw, j, j - 1, j - 2);
   }

   printf ("\n \n segregations at all loci on whole pedigree:");
   for (i = 0; i < pedsize; ++i)
   {
      if (nghd[i].pa->self != UNKNOWN)
      {
         printf ("\n %s", nghd[i].self);
         for (j = 0; j <= nloc; ++j)
            printf ("  %d %d", xx1[j][i], xx0[j][i]);
      }
   }

   printf ("\n \n segregations at marker loci on whole pedigree:");
   for (i = 0; i < pedsize; ++i)
   {
      if (nghd[i].pa->self != UNKNOWN)
      {
         printf ("\n %s", nghd[i].self);
         for (j = 1; j <= nloc; ++j)
            printf ("  %d %d", xx1[j][i], xx0[j][i]);
      }
   }
   fflush (stdout);

/* Now set gene identities (founder gene labels) in all individuals 
   at all marker loci, in accordance with simulated meiosis indicators
 */

   for (j = 0; j <= nloc; ++j)
      ngenes = set_genes (j);
   fflush (stdout);

   printf ("\n \n assigned genes in all observed individuals:");
   for (i = 0; i < NObs; ++i)
   {
      k = ObsIx[i];
      printf ("\n %s", nghd[k].self);
      for (j = 0; j <= nloc; ++j)
         printf ("  %d %d", gg1[j][k], gg0[j][k]);
   }

/* now set the simulated genotypes at marker loci, in accord with original
   data availability
 */
   for (j = 1; j <= nloc; ++j)
      set_genotypes (nfou, j, Qq[j], y[0][j], y[1][j]);

   printf ("\n \n assigned marker genotypes in accordance with data availability:");
   for (i = 0; i < NObs; ++i)
   {
      k = ObsIx[i];
      printf ("\n %s", nghd[k].self);
      for (j = 1; j <= nloc; ++j)
         printf ("  %d %d", gg1[j][k], gg0[j][k]);
   }
   printf ("\n");

   free_xxgg ();
   free_y (&y);

   if (nns)
   {
      free (nns);
      nns = NULL;
   }
   if (qdex)
   {
      free (qdex);
      qdex = NULL;
   }
   if (trtphen)
   {
      free (trtphen);
      trtphen = NULL;
   }

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   /* now we are done  */

   print_tail ();
   fflush (stdout);

   return 0;
}

/** @} */
