/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       parsedefs.h:  header file for the Pars/ library               */
/*                     -  of file scope                                */
/*                     -  included by Pars/ functions                  */
/*                     -  contains definitions of parameter statements */
/*                        and command line paramters for main programs */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEDEFS_H
#define  PARSEDEFS_H

/**
 * @addtogroup parse
 * @{
 */

#define KEYWD 0                         /*Token is a keyword            */
#define PDAT 1                          /*Token is data                 */
#define MAXNKEY  8                      /*The max # of keywords in a    */
                                        /*statement                     */
#define MAXNTOKEN 200                   /*The max # of tokens in a      */
                                        /*statement (init. # only)      */
#define NOMTOKLEN  8                    /*Nominal # of characters per   */
                                        /*token in an input statement   */

#define COM_LIN_FORM  "<prognam> <parfile>"

/*  line_in[]   - the next line of input data                           */
/*  token[][]   - the tokens found in the input statemnt                */
/*  token[]     - a list of pointers to the start of each token         */
/*  tok_typ[]   - corresponding token type                              */
/*                (alphabetic, numeric or string)                       */
/*  aggre_key[] - the aggregate key                                     */
/*                (concatenated first 4 characters of each keyword)     */
/*  key_ix      - the statement type, as determined by the aggregate key*/

static char     line_in[MAXLINLEN+2];
static char  ** token;
static int    * tok_typ;
static char     aggre_key[MAXNKEY*4+1];
static int      key_ix;

/*  function prototypes for functions which correspond to the various   */
/*  input statement types                                               */

/*  for later, note that a typedef can be used;  
 *  try typedef static int PARSTAT_FUNC (void)
 *  PARSTAT_FUNC allow_cmpsiz, allow_obsiz, ...
 */

static int
    not_impl       (void),
    allow_cmpsiz   (void),
    allow_obsiz    (void),
    allow_pedsiz   (void),
    assign_gend    (void),
    assum_all_obs  (void),
    check_eigen    (void),
    check_eig_cmp  (void),
    check_ginv     (void),
    check_gmat     (void),
    check_mc_iter  (void),
    check_trace    (void),
    cmpu_cinbreed  (void),
    cmpu_ckinshp   (void),
    cmpu_scor_iter (void),
    cmpu_c2_loc    (void),
    comp_eigen     (void),
    cmpu_inbreed   (void),
    cmpu_kinshp    (void),
    cmpu_2_loc     (void),
    fit_addi       (void),
    fit_no_gen     (void),
    fit_resi       (void),
    ignor_gend     (void),
    in_eig_fil     (void),
    in_mrkdat_fil  (void),
    in_gend_abs    (void),
    in_gend_pres   (void),
    in_ord_fath    (void),
    in_ord_moth    (void),
    in_obs_abs     (void),
    in_obs_pres    (void),
    in_ped_fil     (void),
    in_ped_int     (void),
    in_ped_int_rl  (void),
    in_ped_n       (void),
    in_ped_n_i     (void),
    in_ped_n_i_r   (void),
    in_ped_n_r     (void),
    in_ped_tr_int  (void),
    in_pedsiz      (void),
    in_peel_fil    (void),
    in_resc_fil    (void),
    in_seed_fil    (void),
    lim_breed      (void),
    lim_em_iter    (void),
    map_cmarkd     (void),
    map_cmarkd_f   (void),
    map_cmarkd_m   (void),
    map_cmarkp     (void),
    map_cmarkp_f   (void),
    map_cmarkp_m   (void),
    map_cmarkr     (void),
    map_cmarkr_f   (void),
    map_cmarkr_m   (void),
    map_ctraitd    (void),
    map_ctraitd_f  (void),
    map_ctraitd_m  (void),
    map_ctraitr    (void),
    map_ctraitr_f  (void),
    map_ctraitr_m  (void),
    map_ctr_begd_f (void),
    map_ctr_begd_m (void),
    map_ctr_begd   (void),
    map_ctr_begr_f (void),
    map_ctr_begr_m (void),
    map_ctr_begr   (void),
    map_ctr_endd_f (void),
    map_ctr_endd_m (void),
    map_ctr_endd   (void),
    map_ctr_endr_f (void),
    map_ctr_endr_m (void),
    map_ctr_endr   (void),
    map_ctr_extd_f (void),
    map_ctr_extd_m (void),
    map_ctr_extd   (void),
    map_ctr_extr_f (void),
    map_ctr_extr_m (void),
    map_ctr_extr   (void),
    map_ctr_intv_f (void),
    map_ctr_antv_f (void),
    map_ctr_intv_m (void),
    map_ctr_antv_m (void),
    map_ctr_intv   (void),
    map_ctr_antv   (void),
    map_markd      (void),
    map_markd_f    (void),
    map_markd_m    (void),
    map_markp      (void),
    map_markp_f    (void),
    map_markp_m    (void),
    map_markr      (void),
    map_markr_f    (void),
    map_markr_m    (void),
    map_traitd     (void),
    map_traitd_f   (void),
    map_traitd_m   (void),
    map_traitr     (void),
    map_traitr_f   (void),
    map_traitr_m   (void),
    map_tr_begd_f  (void),
    map_tr_begd_m  (void),
    map_tr_begd    (void),
    map_tr_begr_f  (void),
    map_tr_begr_m  (void),
    map_tr_begr    (void),
    map_tr_endd_f  (void),
    map_tr_endd_m  (void),
    map_tr_endd    (void),
    map_tr_endr_f  (void),
    map_tr_endr_m  (void),
    map_tr_endr    (void),
    map_tr_extd_f  (void),
    map_tr_extd_m  (void),
    map_tr_extd    (void),
    map_tr_extr_f  (void),
    map_tr_extr_m  (void),
    map_tr_extr    (void),
    map_tr_intv_f  (void),
    map_tr_antv_f  (void),
    map_tr_intv_m  (void),
    map_tr_antv_m  (void),
    map_tr_intv    (void),
    map_tr_antv    (void),
    out_chron      (void),
    out_corr       (void),
    out_covar      (void),
    out_fin_pheno  (void),
    out_mark_seeds (void),
    out_ped_fil    (void),
    out_ord_fath   (void),
    out_found_labs (void),
    out_ord_moth   (void),
    out_tr_vars    (void),
    out_unobs      (void),
    out_rb_est_fil (void),
    out_resc_fil   (void),
    out_scor_fil   (void),
    out_scor_iter  (void),
    out_seed_fil   (void),
    out_space      (void),
    out_trait_seeds(void),
    sampl_by_scan  (void),
    sampl_by_step  (void),
    sela_mark      (void),
    sela_mark_tr   (void),
    sela_cmark     (void),
    sela_cmark_tr  (void),
    sel_cmark      (void),
    sel_cmark_tr   (void),
    sel_mark       (void),
    sel_mark_tr    (void),
    sel_trait      (void),
    sel_tr_eff     (void),
    sel_unlnkd_null(void),
    set_addi_var   (void),
    set_affnam     (void),
    set_affind_tr  (void),
    set_tstpos_wind(void),
    set_br_conv    (void),
    set_burn_iter  (void),
    set_cloc_wind  (void),
    set_cmark_dat  (void),
    set_cmark_frq  (void),
    set_cp_affnam  (void),
    set_cp_prbgam  (void),
    set_pseudo_pr  (void), 
    set_cs_prbgam  (void),
    set_cs_windpat (void),
    set_cp_windpat (void),
    set_eigen      (void),
    set_geno_mean  (void),
    set_lsmpl_prob (void),
    set_loc_wind   (void),
    set_mark_dat   (void),
    set_mark_frq   (void),
    set_mark_seeds (void),
    set_mc_iter    (void),
    set_namd_loc   (void),
    set_norm_frq   (void),
    set_prel_iter  (void),
    set_prbgam     (void),
    set_reco_frq   (void),
    set_resi_var   (void),
    set_sampl_seeds(void),
    set_sc_prbgam  (void),
    set_sc_windpat (void),
    set_sqimp_prop (void),
    set_trait_frq  (void),
    set_trait_seeds(void),
    set_trait_geno (void),
    set_trait_pheno(void),
    set_windpat    (void),
    sim_cmark      (void),
    sim_cmark_trait(void),
    sim_mark       (void),
    sim_mark_trait (void),
    sim_unlinked   (void),
    start_add_cor  (void),
    start_add_cov  (void),
    start_add_var  (void),
    start_effect   (void),
    start_mean     (void),
    start_res_cov  (void),
    start_res_cor  (void),
    start_res_var  (void),
    use_full_em    (void),
    use_part_em    (void),
    use_lsmpl_setup(void),
    use_sqimp_setup(void),
    use_sqimp_realz(void);

/*  prototype for input statement                                             */
/*  proto.key[]:  string containing the first four letters of each            */
/*                keyword in the statement.                                   */
/*  proto.par:    a code for each data value or list of values in the         */
/*                statement.                                                  */
/*                "i" - an integer value             example:   3000          */
/*                "k" - a list of integer values                2 4 5         */
/*                "h" - a hexadecimal value                     0x73331ad7    */
/*                "r" - a real value                            22.33         */
/*                "x" - a list of real values                   .3 .2 .9      */
/*                "s" - a string                                "pedFil3"     */
/*                "n" - a list of names                         A313 8333     */
/*                "g" - alternating names & integers                          */
/*                      (e.g. proband gametes)                  C16 0 D26 1   */
/*                "a" - sets of name plus list of integers,                   */
/*                      with the no. of sets specified by "i"                 */
/*                      (e.g. marker data)                      A1 3 5 3 3    */
/*                                                              B3 3 2 3 3    */

/*  proto.func:   the name of the function which acts upon the data           */
/*                from the statement                                          */
/*  proto.count:  the number of times this statement type occurs in the       */
/*                input file                                                  */

static char *nam_cod = "nga";   /* codes for data lists containing names      */

/* pcods below are:                                                           */
/*       u - univar,     g - unibig,     b - bivar,     m - multivar,         */
/*       d - genedrop,   p - pedchk,     a - lm_auto,   l - lm_lods,          */
/*       s - schnell,    i - ibddrop,    k - kin,       y - lm_bayes,         */
/*       e - lm_markers, r - markerdrop, v - lm_pval                          */


static struct _proto {
       char * key;
       char * par;
       char * pcods;
       int (* func) (void);
       int    count;
               } *ppp;

static struct _proto  proto[] = {
{"allo""comp""size"                              ,"i"    ,"ugbm            ",allow_cmpsiz   ,0},
{"allo""obse""indi"                              ,"i"    ,"ugbm            ",allow_obsiz    ,0},
{"allo""pedi""size"                              ,"i"    ,"ugbmdpalsikyerv ",allow_pedsiz   ,0},
{"assi""gend"                                    ,""     ,"ugbmdpalsikyerv ",assign_gend    ,0},
{"assu""all" "obse"                              ,""     ,"    dp          ",assum_all_obs  ,0},
{"chec""eige"                                    ,""     ,"ugbm            ",check_eigen    ,0},
{"chec""eige""comp"                              ,""     ,"ugbm            ",check_eig_cmp  ,0},
{"chec""ginv"                                    ,""     ," gbm            ",check_ginv     ,0},
{"chec""gmat"                                    ,""     ,"u               ",check_gmat     ,0},
{"chec""prog""mc"  "iter"                        ,"i"    ,"      als  ye v ",check_mc_iter  ,0},
{"chec""trac"                                    ,""     ,"ugbm            ",check_trace    ,0},
{"comp""comp""inbr""coef"                        ,"in"   ,"          k     ",cmpu_cinbreed  ,0},
{"comp""comp""kins""coef"                        ,"in"   ,"          k     ",cmpu_ckinshp   ,0},
{"comp""comp""two-""inbr""coef"                  ,"in"   ,"          k     ",cmpu_c2_loc    ,0},
{"comp""eige"                                    ,""     ," bgm            ",comp_eigen     ,0},
{"comp""inbr""coef"                              ,"n"    ,"          k     ",cmpu_inbreed   ,0},
{"comp""kins""coef"                              ,"n"    ,"          k     ",cmpu_kinshp    ,0},
{"comp""scor""ever""iter"                        ,"i"    ,"      als  ye v ",cmpu_scor_iter ,0},
{"comp""two-""inbr""coef"                        ,"n"    ,"          k     ",cmpu_2_loc     ,0},
{"fit" "addi""corr"                              ,"k"    ,"   m            ",fit_addi       ,0},
{"fit" "addi""cova"                              ,"k"    ,"   m            ",fit_addi       ,0},
{"fit" "envi""mode"                              ,""     ,"   m            ",fit_no_gen     ,0},
{"fit" "resi""corr"                              ,"k"    ,"   m            ",fit_resi       ,0},
{"fit" "resi""cova"                              ,"k"    ,"   m            ",fit_resi       ,0},
{"igno""gend"                                    ,""     ,"ugbmdpalsikyerv ",ignor_gend     ,0},
{"inpu""eige""file"                              ,"s"    ," gbm            ",in_eig_fil     ,0},
{"inpu""mark""data""file"                        ,"s"    ,"      als  yerv ",in_mrkdat_fil  ,0},
{"inpu""pedi""file"                              ,"s"    ,"ugbmdpalsikyerv ",in_ped_fil     ,0},
{"inpu""pedi""reco""fath""moth"                  ,""     ,"ugbmdpalsikyerv ",in_ord_fath    ,0},
{"inpu""pedi""reco""gend""abse"                  ,""     ,"ugbmdpalsikyerv ",in_gend_abs    ,0},
{"inpu""pedi""reco""gend""pres"                  ,""     ,"ugbmdpalsikyerv ",in_gend_pres   ,0},
{"inpu""pedi""reco""inte"                        ,"i"    ,"    dpalsikyerv ",in_ped_int     ,0},
{"inpu""pedi""reco""inte""real"                  ,"ii"   ,"ugbmdpalsikyerv ",in_ped_int_rl  ,0},
{"inpu""pedi""reco""name"                        ,"i"    ,"    dpalsikyerv ",in_ped_n       ,0},
{"inpu""pedi""reco""name""inte"                  ,"ii"   ,"    dpalsikyerv ",in_ped_n_i     ,0},
{"inpu""pedi""reco""name""inte""real"            ,"iii"  ,"ugbmdpalsikyerv ",in_ped_n_i_r   ,0},
{"inpu""pedi""reco""name""real"                  ,"ii"   ,"ugbmdpalsikyerv ",in_ped_n_r     ,0},
{"inpu""pedi""reco""moth""fath"                  ,""     ,"ugbmdpalsikyerv ",in_ord_moth    ,0},
{"inpu""pedi""reco""obse""abse"                  ,""     ,"    dp          ",in_obs_abs     ,0},
{"inpu""pedi""reco""obse""pres"                  ,""     ,"    dp          ",in_obs_pres    ,0},
{"inpu""pedi""reco""trai""inte"                  ,"kk"   ,"      als  yerv ",in_ped_tr_int  ,0},
{"inpu""pedi""size"                              ,"i"    ,"ugbmdpalsikyerv ",in_pedsiz      ,0},
{"inpu""peel""sequ""file"                        ,"s"    ,"u               ",in_peel_fil    ,0},
{"inpu""resc""file"                              ,"s"    ,"      al        ",in_resc_fil    ,0},
{"inpu""seed""file"                              ,"s"    ,"    d alsi yerv ",in_seed_fil    ,0},
{"limi""bree""iter"                              ,"i"    ,"ugbm            ",lim_breed      ,0},
{"limi""em"  "iter"                              ,"i"    ,"ugbm            ",lim_em_iter    ,0},
{"map" "chro""gend""f"   "mark""dist"            ,"ix"   ,"    d alsi yerv ",map_cmarkd_f   ,0},
{"map" "chro""gend""f"   "mark""posi"            ,"ix"   ,"    d alsi yerv ",map_cmarkp_f   ,0},
{"map" "chro""gend""f"   "mark""reco""frac"      ,"ix"   ,"    d alsi yerv ",map_cmarkr_f   ,0},
{"map" "chro""gend""f"   "trai""all" "inte""prop","iix"  ,"       ls  ye   ",map_ctr_antv_f ,0},
{"map" "chro""gend""f"   "trai""begi""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_begd_f ,0},
{"map" "chro""gend""f"   "trai""begi""reco""frac","iix"  ,"       ls  ye   ",map_ctr_begr_f ,0},
{"map" "chro""gend""f"   "trai""endi""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_endd_f ,0},
{"map" "chro""gend""f"   "trai""endi""reco""frac","iix"  ,"       ls  ye   ",map_ctr_endr_f ,0},
{"map" "chro""gend""f"   "trai""exte""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_extd_f ,0},
{"map" "chro""gend""f"   "trai""exte""reco""frac","iix"  ,"       ls  ye   ",map_ctr_extr_f ,0},
{"map" "chro""gend""f"   "trai""inte""prop"      ,"iikx" ,"       ls  ye   ",map_ctr_intv_f ,0},
{"map" "chro""gend""f"   "trai""mark""dist"      ,"ikkx" ,"    d a  i   r  ",map_ctraitd_f  ,0},
{"map" "chro""gend""f"   "trai""mark""reco""frac","ikkx" ,"    d a  i   r  ",map_ctraitr_f  ,0},
{"map" "chro""gend""m"   "mark""dist"            ,"ix"   ,"    d alsi yerv ",map_cmarkd_m   ,0},
{"map" "chro""gend""m"   "mark""posi"            ,"ix"   ,"    d alsi yerv ",map_cmarkp_m   ,0},
{"map" "chro""gend""m"   "mark""reco""frac"      ,"ix"   ,"    d alsi yerv ",map_cmarkr_m   ,0},
{"map" "chro""gend""m"   "trai""all" "inte""prop","iix"  ,"       ls  ye   ",map_ctr_antv_m ,0},
{"map" "chro""gend""m"   "trai""begi""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_begd_m ,0},
{"map" "chro""gend""m"   "trai""begi""reco""frac","iix"  ,"       ls  ye   ",map_ctr_begr_m ,0},
{"map" "chro""gend""m"   "trai""endi""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_endd_m ,0},
{"map" "chro""gend""m"   "trai""endi""reco""frac","iix"  ,"       ls  ye   ",map_ctr_endr_m ,0},
{"map" "chro""gend""m"   "trai""exte""dist"      ,"iix"  ,"       ls  ye   ",map_ctr_extd_m ,0},
{"map" "chro""gend""m"   "trai""exte""reco""frac","iix"  ,"       ls  ye   ",map_ctr_extr_m ,0},
{"map" "chro""gend""m"   "trai""inte""prop"      ,"iikx" ,"       ls  ye   ",map_ctr_intv_m ,0},
{"map" "chro""gend""m"   "trai""mark""dist"      ,"ikkx" ,"    d a  i   r  ",map_ctraitd_m  ,0},
{"map" "chro""gend""m"   "trai""mark""reco""frac","ikkx" ,"    d a  i   r  ",map_ctraitr_m  ,0},
{"map" "chro""mark""dist"                        ,"ix"   ,"    d alsi yerv ",map_cmarkd     ,0},
{"map" "chro""mark""posi"                        ,"ix"   ,"    d alsi yerv ",map_cmarkp     ,0},
{"map" "chro""mark""reco""frac"                  ,"ix"   ,"    d alsi yerv ",map_cmarkr     ,0},
{"map" "chro""trai""all" "inte""prop"            ,"iix"  ,"       ls  ye   ",map_ctr_antv   ,0},
{"map" "chro""trai""begi""dist"                  ,"iix"  ,"       ls  ye   ",map_ctr_begd   ,0},
{"map" "chro""trai""begi""reco""frac"            ,"iix"  ,"       ls  ye   ",map_ctr_begr   ,0},
{"map" "chro""trai""endi""dist"                  ,"iix"  ,"       ls  ye   ",map_ctr_endd   ,0},
{"map" "chro""trai""endi""reco""frac"            ,"iix"  ,"       ls  ye   ",map_ctr_endr   ,0},
{"map" "chro""trai""exte""dist"                  ,"iix"  ,"       ls  ye   ",map_ctr_extd   ,0},
{"map" "chro""trai""exte""reco""frac"            ,"iix"  ,"       ls  ye   ",map_ctr_extr   ,0},
{"map" "chro""trai""inte""prop"                  ,"iikx" ,"       ls  ye   ",map_ctr_intv   ,0},
{"map" "chro""trai""mark""dist"                  ,"ikkx" ,"    d a  i   r  ",map_ctraitd    ,0},
{"map" "chro""trai""mark""reco""frac"            ,"ikkx" ,"    d a  i   r  ",map_ctraitr    ,0},
{"map" "gend""f"   "mark""dist"                  ,"x"    ,"    d alsi yerv ",map_markd_f    ,0},
{"map" "gend""f"   "mark""posi"                  ,"x"    ,"    d alsi yerv ",map_markp_f    ,0},
{"map" "gend""f"   "mark""reco""frac"            ,"x"    ,"    d alsi yerv ",map_markr_f    ,0},
{"map" "gend""f"   "trai""all" "inte""prop"      ,"ix"   ,"       ls  ye   ",map_tr_antv_f  ,0},
{"map" "gend""f"   "trai""begi""dist"            ,"ix"   ,"       ls  ye   ",map_tr_begd_f  ,0},
{"map" "gend""f"   "trai""begi""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_begr_f  ,0},
{"map" "gend""f"   "trai""endi""dist"            ,"ix"   ,"       ls  ye   ",map_tr_endd_f  ,0},
{"map" "gend""f"   "trai""endi""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_endr_f  ,0},
{"map" "gend""f"   "trai""exte""dist"            ,"ix"   ,"       ls  ye   ",map_tr_extd_f  ,0},
{"map" "gend""f"   "trai""exte""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_extr_f  ,0},
{"map" "gend""f"   "trai""inte""prop"            ,"ikx"  ,"       ls  ye   ",map_tr_intv_f  ,0},
{"map" "gend""f"   "trai""mark""dist"            ,"kkx"  ,"    d a  i   r  ",map_traitd_f   ,0},
{"map" "gend""f"   "trai""mark""reco""frac"      ,"kkx"  ,"    d a  i   r  ",map_traitr_f   ,0},
{"map" "gend""m"   "mark""dist"                  ,"x"    ,"    d alsi yerv ",map_markd_m    ,0},
{"map" "gend""m"   "mark""posi"                  ,"x"    ,"    d alsi yerv ",map_markp_m    ,0},
{"map" "gend""m"   "mark""reco""frac"            ,"x"    ,"    d alsi yerv ",map_markr_m    ,0},
{"map" "gend""m"   "trai""all" "inte""prop"      ,"ix"   ,"       ls  ye   ",map_tr_antv_m  ,0},
{"map" "gend""m"   "trai""begi""dist"            ,"ix"   ,"       ls  ye   ",map_tr_begd_m  ,0},
{"map" "gend""m"   "trai""begi""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_begr_m  ,0},
{"map" "gend""m"   "trai""endi""dist"            ,"ix"   ,"       ls  ye   ",map_tr_endd_m  ,0},
{"map" "gend""m"   "trai""endi""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_endr_m  ,0},
{"map" "gend""m"   "trai""exte""dist"            ,"ix"   ,"       ls  ye   ",map_tr_extd_m  ,0},
{"map" "gend""m"   "trai""exte""reco""frac"      ,"ix"   ,"       ls  ye   ",map_tr_extr_m  ,0},
{"map" "gend""m"   "trai""inte""prop"            ,"ikx"  ,"       ls  ye   ",map_tr_intv_m  ,0},
{"map" "gend""m"   "trai""mark""dist"            ,"kkx"  ,"    d a  i   r  ",map_traitd_m   ,0},
{"map" "gend""m"   "trai""mark""reco""frac"      ,"kkx"  ,"    d a  i   r  ",map_traitr_m   ,0},
{"map" "mark""dist"                              ,"x"    ,"    d alsi yerv ",map_markd      ,0},
{"map" "mark""posi"                              ,"x"    ,"    d alsi yerv ",map_markp      ,0},
{"map" "mark""reco""frac"                        ,"x"    ,"    d alsi yerv ",map_markr      ,0},
{"map" "trai""all" "inte""prop"                  ,"ix"   ,"       ls  ye   ",map_tr_antv    ,0},
{"map" "trai""begi""dist"                        ,"ix"   ,"       ls  ye   ",map_tr_begd    ,0},
{"map" "trai""begi""reco""frac"                  ,"ix"   ,"       ls  ye   ",map_tr_begr    ,0},
{"map" "trai""endi""dist"                        ,"ix"   ,"       ls  ye   ",map_tr_endd    ,0},
{"map" "trai""endi""reco""frac"                  ,"ix"   ,"       ls  ye   ",map_tr_endr    ,0},
{"map" "trai""exte""dist"                        ,"ix"   ,"       ls  ye   ",map_tr_extd    ,0},
{"map" "trai""exte""reco""frac"                  ,"ix"   ,"       ls  ye   ",map_tr_extr    ,0},
{"map" "trai""inte""prop"                        ,"ikx"  ,"       ls  ye   ",map_tr_intv    ,0},
{"map" "trai""mark""dist"                        ,"kkx"  ,"    d a  i   r  ",map_traitd     ,0},
{"map" "trai""mark""reco""frac"                  ,"kkx"  ,"    d a  i   r  ",map_traitr     ,0},
{"outp""fina""adju""phen"                        ,""     ,"ugbm            ",out_fin_pheno  ,0},
{"outp""mark""seed""only"                        ,""     ,"    d           ",out_mark_seeds ,0},
{"outp""pedi""chro"                              ,""     ,"    dp          ",out_chron      ,0},
{"outp""pedi""file"                              ,"s"    ,"ugbmdpalsikyerv ",out_ped_fil    ,0},
{"outp""pedi""reco""fath""moth"                  ,""     ,"    dp          ",out_ord_fath   ,0},
{"outp""pedi""reco""foun""gene""labe"            ,""     ,"    d           ",out_found_labs ,0},
{"outp""pedi""reco""moth""fath"                  ,""     ,"    dp          ",out_ord_moth   ,0},
{"outp""pedi""reco""trai""late""vari"            ,""     ,"    d           ",out_tr_vars    ,0},
{"outp""pedi""reco""unob""vari"                  ,""     ,"    d           ",out_unobs      ,0},
{"outp""rao-""esti""file"                        ,"s"    ,"           y    ",out_rb_est_fil ,0},
{"outp""resc""file"                              ,"s"    ,"      al        ",out_resc_fil   ,0},
{"outp""scor""file"                              ,"s"    ,"      a         ",out_scor_fil   ,0},
{"outp""scor""mc"  "iter"                        ,"i"    ,"      a         ",out_scor_iter  ,0},
{"outp""seed""file"                              ,"s"    ,"    d alsi yerv ",out_seed_fil   ,0},
{"outp""spac""em"  "iter"                        ,"i"    ,"ugbm            ",out_space      ,0},
{"outp""stat""corr"                              ,""     ,"   m            ",out_corr       ,0},
{"outp""stat""cova"                              ,""     ,"  bm            ",out_covar      ,0},
{"outp""trai""seed""only"                        ,""     ,"    d           ",out_trait_seeds,0},
{"samp""by"  "scan"                              ,""     ,"      als  ye v ",sampl_by_scan  ,0},
{"samp""by"  "step"                              ,""     ,"      als   e v ",sampl_by_step  ,0},
{"sele""all" "mark"                              ,""     ,"      a s     v ",sela_mark      ,0},
{"sele""all" "mark""trai"                        ,"i"    ,"      als  ye   ",sela_mark_tr   ,0},
{"sele""chro""all" "mark"                        ,"i"    ,"      a s     v ",sela_cmark     ,0},
{"sele""chro""all" "mark""trai"                  ,"ii"   ,"      als  ye   ",sela_cmark_tr  ,0},
{"sele""chro""mark"                              ,"ik"   ,"      a s     v ",sel_cmark      ,0},
{"sele""chro""mark""trai"                        ,"iki"  ,"      als  ye   ",sel_cmark_tr   ,0},
{"sele""mark"                                    ,"k"    ,"      a s     v ",sel_mark       ,0},
{"sele""mark""trai"                              ,"ki"   ,"      als  ye   ",sel_mark_tr    ,0},
{"sele""trai"                                    ,"i"    ,"ugbm            ",sel_trait      ,0},
{"sele""trai""effe"                              ,"ik"   ,"ugbm            ",sel_tr_eff     ,0},
{"sele""unli""trai"                              ,"k"    ,"      a         ",sim_unlinked   ,0},
{"sele""unli""null""locu"                        ,""     ,"              v ",sel_unlnkd_null,0},
{"set" "affe""indi"                              ,"n"    ,"              v ",set_affnam     ,0},
{"set" "affe""indi""trai"                        ,"i"    ,"              v ",set_affind_tr  ,0},
{"set" "bree""conv"                              ,"r"    ,"ugbm            ",set_br_conv    ,0},
{"set" "burn""iter"                              ,"i"    ,"      al   ye v ",set_burn_iter  ,0},
{"set" "chro""locu""wind"                        ,"ii"   ,"      a  i      ",set_cloc_wind  ,0},
{"set" "chro""mark""data"                        ,"iia"  ,"      als  yerv ",set_cmark_dat  ,0},
{"set" "chro""mark""freq"                        ,"ikx"  ,"    d als  yerv ",set_cmark_frq  ,0},
{"set" "comp""affe""indi"                        ,"in"   ,"              v ",set_cp_affnam  ,0},
{"set" "comp""prob""game"                        ,"ig"   ,"      a  i      ",set_cp_prbgam  ,0},
{"set" "pseu"                                    ,"x"    ,"           y    ",set_pseudo_pr  ,0}, 
{"set" "comp""scor""prob""game"                  ,"iig"  ,"      a  i      ",set_cs_prbgam  ,0},
{"set" "comp""scor""wind""patt"                  ,"iik"  ,"      a         ",set_cs_windpat ,0},
{"set" "comp""wind""patt"                        ,"ik"   ,"      a         ",set_cp_windpat ,0},
{"set" "eige"                                    ,"x"    ," gbm            ",set_eigen      ,0},
{"set" "l-sa""prob"                              ,"r"    ,"      als  ye v ",set_lsmpl_prob ,0},
{"set" "locu""wind"                              ,"i"    ,"      a  i      ",set_loc_wind   ,0},
{"set" "mark""data"                              ,"ia"   ,"      als  yerv ",set_mark_dat   ,0},
{"set" "mark""freq"                              ,"kx"   ,"    d als  yerv ",set_mark_frq   ,0},
{"set" "mark""seed"                              ,"hh"   ,"    d           ",set_mark_seeds ,0},
{"set" "mc"  "iter"                              ,"i"    ,"      alsi ye v ",set_mc_iter    ,0},
{"set" "name""loci"                              ,""     ,"      als  ye v ",set_namd_loc   ,0},
{"set" "norm""freq"                              ,""     ,"    d als  yerv ",set_norm_frq   ,0},
{"set" "peel""seq"                               ,"x"    ,""                ,not_impl       ,0},
{"set" "prel""iter"                              ,"i"    ,"           y    ",set_prel_iter  ,0},
{"set" "prob""game"                              ,"g"    ,"      a  i      ",set_prbgam     ,0},
{"set" "reco""freq"                              ,"x"    ,"          k     ",set_reco_frq   ,0},
{"set" "resi""corr"                              ,""     ,"    d           ",not_impl       ,0},
{"set" "resi""cova"                              ,""     ,"    d           ",not_impl       ,0},
{"set" "samp""seed"                              ,"hh"   ,"      alsi yerv ",set_sampl_seeds,0},
{"set" "scor""prob""game"                        ,"ig"   ,"      a  i      ",set_sc_prbgam  ,0},
{"set" "scor""wind""patt"                        ,"ik"   ,"      a         ",set_sc_windpat ,0},
{"set" "sequ""impu""prop""ever""iter"            ,"i"    ,"           y    ",set_sqimp_prop ,0},
{"set" "test""posi""wind"                        ,"i"    ,"           y    ",set_tstpos_wind,0},
{"set" "trai""addi""vari"                        ,"kr"   ,"    d   s   e   ",set_addi_var   ,0},
{"set" "trai""data""geno"                        ,""     ,"      als  ye v ",set_trait_geno, 0},
{"set" "trai""data""phen"                        ,""     ,"      als  ye v ",set_trait_pheno,0},
{"set" "trai""freq"                              ,"kx"   ,"    d als  ye   ",set_trait_frq  ,0},
{"set" "trai""geno""mean"                        ,"kx"   ,"    d   s   e   ",set_geno_mean  ,0},
{"set" "trai""resi""vari"                        ,"kr"   ,"    d   s   e   ",set_resi_var   ,0},
{"set" "trai""seed"                              ,"hh"   ,"    d           ",set_trait_seeds,0},
{"set" "wind""patt"                              ,"k"    ,"      a         ",set_windpat    ,0},
{"simu""chro""mark"                              ,"ii"   ,"    d    i   r  ",sim_cmark      ,0},
{"simu""chro""mark""trai"                        ,"iii"  ,"    d    i      ",sim_cmark_trait,0},
{"simu""mark"                                    ,"i"    ,"    d    i   r  ",sim_mark       ,0},
{"simu""mark""trai"                              ,"ii"   ,"    d    i      ",sim_mark_trait ,0},
{"simu""unli""trai"                              ,"k"    ,"    d    i      ",sim_unlinked   ,0},
{"star""addi""corr"                              ,"x"    ,"  bm            ",start_add_cor  ,0},
{"star""addi""cova"                              ,"x"    ,"  bm            ",start_add_cov  ,0},
{"star""addi""vari"                              ,"x"    ,"ugbm            ",start_add_var  ,0},
{"star""resi""corr"                              ,"x"    ,"  bm            ",start_res_cor  ,0},
{"star""resi""cova"                              ,"x"    ,"  bm            ",start_res_cov  ,0},
{"star""resi""vari"                              ,"x"    ,"ugbm            ",start_res_var  ,0},
{"star""trai""effe"                              ,"iix"  ,"ugbm            ",start_effect   ,0},
{"star""trai""mean"                              ,"ir"   ,"ugbm            ",start_mean     ,0},
{"use" "full""em"                                ,""     ,"   m            ",use_full_em    ,0},
{"use" "locu""samp""for" "setu"                  ,""     ,"      als  ye v ",use_lsmpl_setup,0},
{"use" "part""em"                                ,""     ,"   m            ",use_part_em    ,0},
{"use" "sequ""impu""for" "setu"                  ,""     ,"      als  ye v ",use_sqimp_setup,0},
{"use" "sequ""impu""real""for" "setu"            ,"i"    ,"      als  ye v ",use_sqimp_realz,0},
{""                                              ,""     ,""                ,not_impl       ,0}};


/*  descriptions of input statements, SAME ORDER and SAME NUMBER        */
/*  as the statement prototypes above                                   */

static char * input_statement[] = {
        "allow component size  I"                                         ,
        "allow observed individuals  I"                                   ,
        "allow pedigree size  I"                                          ,
        "assign gender"                                                   ,
        "assume all observed"                                             ,
        "check eigenvalues"                                               ,
        "check eigenvalue computation"                                    ,
        "check ginverse"                                                  ,
        "check gmatrix"                                                   ,
        "check progress MC iterations  I"                                 ,
        "check trace"                                                     ,
        "compute component  J  inbreeding coefficients  N1 N2 ..."        ,
        "compute component  J  kinship coefficients  N1A N1B  N2A N2B ...",
        "compute component  J  two-locus inbreeding coefficients  N1 ..." ,
        "compute eigenvalues"                                             ,
        "compute inbreeding coefficients  N1 N2 ..."                      ,
        "compute kinship coefficients  N1A N1B  N2A N2B ..."              ,
        "compute scores every  I  iterations"                             ,
        "compute two-locus inbreeding coefficients  N1 ..."               ,
        "fit additive correlations  K12 ..."                              ,
        "fit additive covariances  K12 ..."                               ,
        "fit environmental model"                                         ,
        "fit residual correlations  K12 ..."                              ,
        "fit residual covariances  K12 ..."                               ,
        "ignore gender"                                                   ,
        "input eigenvalue file  S"                                        ,
        "input marker data file  S"                                       ,
        "input pedigree file  S"                                          ,
        "input pedigree record father mother"                             ,
        "input pedigree record gender absent"                             ,
        "input pedigree record gender present"                            ,
        "input pedigree record integers  J"                               ,
        "input pedigree record integers  J  reals  K"                     ,
        "input pedigree record names  I"                                  ,
        "input pedigree record names  I  integers  J"                     ,
        "input pedigree record names  I  integers  J  reals  K"           ,
        "input pedigree record names  I  reals  K"                        ,
        "input pedigree record mother father"                             ,
        "input pedigree record observed absent"                           ,
        "input pedigree record observed present"                          ,
        "input pedigree record traits K1 K2 ...  integers I1 I2 ..."      ,
        "input pedigree size  I"                                          ,
        "input peeling sequence file  S"                                  ,
        "input rescue file  S"                                            ,
        "input seed file  S"                                              ,
        "limit breeding iterations  I"                                    ,
        "limit EM iterations  I"                                          ,
        "map chromosome  I  gender F  marker distances  X1 ..."           ,
        "map chromosome  I  gender F  marker positions  X1 ..."           ,
        "map chromosome  I  gender F  marker "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  all interval "
                               "proportions  X1 ..."                      ,
        "map chromosome  I  gender F  trait  J  "
                                            "beginning distances  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                              "beginning recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                                               "ending distances  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                                 "ending recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                                             "external distances  X1 ..." ,
        "map chromosome  I  gender F  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  gender F  trait  J  intervals  K1 ...  "
                               "proportions  X1 ..."                      ,
        "map chromosome  I  gender F  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  gender F  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  marker distances  X1 ..."           ,
        "map chromosome  I  gender M  marker positions  X1 ..."           ,
        "map chromosome  I  gender M  marker "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  all interval "
                               "proportions  X1 ..."                      ,
        "map chromosome  I  gender M  trait  J  "
                                            "beginning distances  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                              "beginning recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                                               "ending distances  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                                 "ending recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                                             "external distances  X1 ..." ,
        "map chromosome  I  gender M  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  gender M  trait  J  intervals   K1 ...  "
                               "proportions  X1 ..."                      ,
        "map chromosome  I  gender M  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  gender M  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map chromosome  I  marker distances  X1 ..."                     ,
        "map chromosome  I  marker positions  X1 ..."                     ,
        "map chromosome  I  marker recombination fractions  X1 ..."       ,
        "map chromosome  I  trait  J  all interval proportions  X1 ..."   ,
        "map chromosome  I  trait  J  beginning distances  X1 ..."        ,
        "map chromosome  I  trait  J  "
                               "beginning recombination fractions  X1 ...",
        "map chromosome  I  trait  J  ending distances  X1 ..."           ,
        "map chromosome  I  trait  J  "
                               "ending recombination fractions  X1 ..."   ,
        "map chromosome  I  trait  J  external distances  X1 ..."         ,
        "map chromosome  I  trait  J  "
                               "external recombination fractions  X1 ..." ,
        "map chromosome  I  trait  J  intervals  K1 ...  "
                               "proportions  X1 ..."                      ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  "
                                                      "distances  X1 ..." ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map gender F  marker distances  X1 ..."                          ,
        "map gender F  marker positions  X1 ..."                          ,
        "map gender F  marker recombination fractions  X1 ..."            ,
        "map gender F  trait  J  all interval proportions  X1 ..."        ,
        "map gender F  trait  J  beginning distances  X1 ..."             ,
        "map gender F  trait  J  "
                               "beginning recombination fractions  X1 ...",
        "map gender F  trait  J  ending distances  X1 ..."                ,
        "map gender F  trait  J  ending recombination fractions  X1 ..."  ,
        "map gender F  trait  J  external distances  X1 ..."              ,
        "map gender F  trait  J  external recombination fractions  X1 ...",
        "map gender F  trait  J  intervals  K1 ...  proportions  X1 ..."  ,
        "map gender F  traits  J1 ...  markers  K1 ...  distances  X1 ...",
        "map gender F  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map gender M  marker distances  X1 ..."                          ,
        "map gender M  marker positions  X1 ..."                          ,
        "map gender M  marker recombination fractions  X1 ..."            ,
        "map gender M  trait  J  all interval proportions  X1 ..."        ,
        "map gender M  trait  J  beginning distances  X1 ..."             ,
        "map gender M  trait  J  "
                               "beginning recombination fractions  X1 ...",
        "map gender M  trait  J  ending distances  X1 ..."                ,
        "map gender M  trait  J  ending recombination fractions  X1 ..."  ,
        "map gender M  trait  J  external distances  X1 ..."              ,
        "map gender M  trait  J  external recombination fractions  X1 ...",
        "map gender M  trait  J  intervals  K1 ...  proportions  X1 ..."  ,
        "map gender M  traits  J1 ...  markers  K1 ...  distances  X1 ...",
        "map gender M  traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "map marker distances  X1 ..."                                    ,
        "map marker positions  X1 ..."                                    ,
        "map marker recombination fractions  X1 ..."                      ,
        "map trait  J  all interval proportions  X1 ..."                  ,
        "map trait  J  beginning distances  X1 ..."                       ,
        "map trait  J  beginning recombination fractions  X1 ..."         ,
        "map trait  J  ending distances  X1 ..."                          ,
        "map trait  J  ending recombination fractions  X1 ..."            ,
        "map trait  J  external distances  X1 ..."                        ,
        "map trait  J  external recombination fractions  X1 ..."          ,
        "map trait  J  intervals  K1 ...  proportions  X1 ..."            ,
        "map traits  J1 ...  markers  K1 ...  distances  X1 ..."          ,
        "map traits  J1 ...  markers  K1 ...  "
                                        "recombination fractions  X1 ..." ,
        "output final adjusted phenotypes"                                ,
        "output marker seeds only"                                        ,
        "output pedigree chronological"                                   ,
        "output pedigree file  S"                                         ,
        "output pedigree record father mother"                            ,
        "output pedigree record founder gene labels"                      ,
        "output pedigree record mother father"                            ,
        "output pedigree record trait latent variables"                   ,
        "output pedigree record unobserved variables"                     ,
        "output rao-blackwellized estimates file  S"                      ,
        "output rescue file  S"                                           ,
        "output scores file  S"                                           ,
        "output scores MC iterations  I"                                  ,
        "output seed file  S"                                             ,
        "output spacing  I  EM iterations"                                ,
        "output statistics correlations"                                  ,
        "output statistics covariances"                                   ,
        "output trait seeds only"                                         ,
        "sample by scan"                                                  ,
        "sample by step"                                                  ,
        "select all markers"                                              ,
        "select all markers  traits K"                                    ,
        "select chromosome  I  all markers"                               ,
        "select chromosome  I  all markers  traits  K"                    ,
        "select chromosome  I  markers  J1 J2"                            ,
        "select chromosome  I  markers  J1 J2 ...  traits  K"             ,
        "select markers  J1 J2 ..."                                       ,
        "select markers  J1 J2 ...  traits  K"                            ,
        "select trait  I"                                                 ,
        "select trait  I  effects  J1 ..."                                ,
        "select unlinked null locus"                                      ,
        "select unlinked traits  K1 ..."                                  ,
        "set affected individuals  N1 N2 ..."                             ,
        "set affected individuals  trait K"                               ,
        "set breeding convergence  X"                                     ,
        "set burn-in iterations  I"                                       ,
        "set chromosome  I  locus window  J"                              ,
        "set chromosome  I  markers  J  data  N1  M1 M2 ..."              ,
        "set chromosome  I  markers  K1 ...  frequencies  X1 ... "        ,
        "set component  J  affected individuals  N1 N2 ..."               ,
        "set component  J  proband gametes  N1 K1  N2 K2 ..."             ,
        "set component  J  scoreset  K  proband gametes  N1 K1  N2 K2 ...",
        "set component  J  scoreset  K  window patterns  L1 ..."          ,
        "set component  J  window patterns  L1 ..."                       ,
        "set eigenvalues  X1 ..."                                         ,
        "set L-sampler probability  X"                                    ,
        "set locus window  J"                                             ,
        "set markers  J  data  N1  M1 M2 ..."                             ,
        "set markers  K1 ...  frequencies  X1 ..."                        ,
        "set marker seeds  H1  H2"                                        ,
        "set MC iterations  I"                                            ,
        "set named loci"                                                  ,
        "set normalized frequencies"                                      ,
        "set peeling sequence  X1 ..."                                    ,
        "set preliminary iterations  I"                                   ,
        "set proband gametes  N1 K1  N2 K2 ..."                           ,
        "set recombination frequencies X1 ..."                            ,
        "set residual correlations  X12 ..."                              ,
        "set residual covariances  X12 ..."                               ,
        "set sampler seeds  H1  H2"                                       ,
        "set scoreset  K  proband gametes  N1 K1  N2 K2 ..."              ,
        "set scoreset  K  window patterns  L1 ..."                        ,
        "set sequential imputation proposals every  I  iterations"        ,
        "set test position window  I"                                     ,
        "set traits  J1 ...  additive variances  X1 ..."                  ,
        "set trait data genotype"                                         ,
        "set trait data phenotype"                                        ,
        "set traits  J1 ...  frequencies  X1 ..."                         ,
        "set traits  J1 ...  genotype means  X1 ..."                      ,
        "set traits  J1 ... residual variances  X1 ..."                   ,
        "set trait seeds  H1  H2"                                         ,
        "set window patterns  L1 ..."                                     ,
        "simulate chromosome  I  markers  J"                              ,
        "simulate chromosome  I  markers  J  traits  K"                   ,
        "simulate markers  J"                                             ,
        "simulate markers  J  traits  K"                                  ,
        "simulate unlinked traits  K1 ..."                                ,
        "start additive correlations  X12 ..."                            ,
        "start additive covariances  X12 ..."                             ,
        "start additive variances  X1 ..."                                ,
        "start residual correlations  X12 ..."                            ,
        "start residual covariances  X12 ..."                             ,
        "start residual variances  X1 ..."                                ,
        "start trait  I  effect  J  X1 ..."                               ,
        "start trait  I  mean  X"                                         ,
        "use full EM"                                                     ,
        "use locus-by-locus sampling for setup"                           ,
        "use partitioned EM"                                              ,
        "use sequential imputation for setup"                             ,
        "use  I  sequential imputation realizations for setup"            ,
        ""                                                              } ;

/* these words may begin a new input statement, thus delimiting the     */
/* previous statement.                                                  */

static char * verb[] = {
             "allo",                /* allow        */
             "assi",                /* assign       */ 
             "assu",                /* assume       */
             "chec",                /* check        */
             "comp",                /* compute      */
             "fit",                 /* fit          */
             "igno",                /* ignore       */
             "inpu",                /* input        */
             "limi",                /* limit        */
             "map",                 /* map          */
             "outp",                /* output       */
             "samp",                /* sample       */
             "sele",                /* select       */
             "set",                 /* set          */
             "simu",                /* simulate     */
             "star",                /* start        */
             "use",                 /* use          */
             "****",                /* end-of-file  */
             ""     };

/** @} */

#endif  /*PARSEDEFS_H*/

