/* $Id: g4link.c,v 1.7 2002/09/10 00:23:52 nali Exp $ */

#include "nghds.h"
/* g4link provides interface similiar to kinship () */
double g4link (Ind * inda, int ga,
               Ind * indb, int gb,
               Ind * indc, int gc,
               Ind * indd, int gd, double r)
{
   /* proband indices */
   int   idx_a, idx_b, idx_c, idx_d;
   /* parents indices */
   int   idx_pa, idx_pb, idx_pc, idx_pd;

   idx_a = inda->index;
   if (ga == 0)
      idx_pa = inda->ma->index;
   else
      idx_pa = inda->pa->index;

   idx_b = indb->index;
   if (gb == 0)
      idx_pb = indb->ma->index;
   else
      idx_pb = indb->pa->index;

   idx_c = indc->index;
   if (gc == 0)
      idx_pc = indc->ma->index;
   else
      idx_pc = indc->pa->index;

   idx_d = indd->index;
   if (gd == 0)
      idx_pd = indd->ma->index;
   else
      idx_pd = indd->pa->index;

   return g4link_idx (idx_pa, idx_pb, idx_pc, idx_pd,
                      idx_a, idx_b, idx_c, idx_d, r);
}

/* g4link_idx is called through indices and is used for recursion */
double g4link_idx (int a,                   /* a is one parent of ta, and so on */
                   int b,
                   int c,
                   int d,
                   int ta,                  /* Four individuals (could be the same) */
                   int tb,
                   int tc,
                   int td,
                   double r                 /* Recombination fraction between loci */
)
/* recursive program for identity at two linked loci  */
{
   int   aa, bb, cc, dd, mm, ff, taa, tbb, tcc, tdd;
   double xx = 0.0;

   double q = 1.0 - r;
   double rr = 0.5 - r * q;

   if (isNullNode (&nghd[a]) ||
       isNullNode (&nghd[b]) ||
       isNullNode (&nghd[c]) ||
       isNullNode (&nghd[d]))           /* One of them is a founder */
   {
      return 0.0;
   }

   aa = max (a, b);
   bb = min (a, b);
   cc = max (c, d);
   dd = min (c, d);

   if (aa == a)
   {
      taa = ta;
      tbb = tb;
   }
   else
   {
      taa = tb;
      tbb = ta;
   }

   if (cc == c)
   {
      tcc = tc;
      tdd = td;
   }
   else
   {
      tcc = td;
      tdd = tc;
   }

   if (aa < cc)
   {
      aa = cc;
      taa = tcc;
      bb = dd;
      tbb = tdd;
      cc = max (a, b);
      dd = min (a, b);
      if (cc == a)
      {
         tcc = ta;
         tdd = tb;
      }
      else
      {
         tcc = tb;
         tdd = ta;
      }
   }

   mm = nghd[aa].ma->index;
   ff = nghd[aa].pa->index;

   /* all now set so aa is expandee */
   if (aa == cc)
   {
      /*   if (isNullNode (&nghd[mm]) && isNullNode (&nghd[ff])) */
      if (isNullNode (nghd[aa].ma) && isNullNode (nghd[aa].pa))
      {
         if ((aa != bb) || (aa != dd))
         {
            return 0.0;
         }
         else
         {
            if (((taa == tcc) && (tbb == tdd)) || ((taa == tdd) && (tbb == tcc)))
            {
               return rr;
            }
            else
            {
               return 0.25;
            }
         }                              /* founder expandee a */
      }
      else if ((aa == bb) && (aa == dd))
      {
         if (((taa == tcc) && (tbb == tdd)) || ((taa == tdd) && (tbb == tcc)))
         {
            xx = (rr * (1.0 + g4link_idx (mm, ff, mm, ff, aa, aa, aa, aa, r))
                  + 2.0 * r * q * kinship (nghd + mm, nghd + ff));
         }
         else
         {
            xx = (0.25 * (1.0 + g4link_idx (mm, ff, mm, ff, aa, aa, aa, aa, r))
                  + 0.5 * kinship (nghd + mm, nghd + ff));
         }
         return xx;
      }
      else if ((aa != bb) && (aa != dd))
      {
         if (taa == tcc)
         {
            xx = 0.5 * (q * (g4link_idx (mm, bb, mm, dd, aa, tbb, aa, tdd, r) +
                           g4link_idx (ff, bb, ff, dd, aa, tbb, aa, tdd, r)) +
                        r * (g4link_idx (mm, bb, ff, dd, aa, tbb, aa, tdd, r) +
                           g4link_idx (ff, bb, mm, dd, aa, tbb, aa, tdd, r)));
         }
         else
         {
            xx = 0.25 * (g4link_idx (mm, bb, mm, dd, aa, tbb, aa, tdd, r) +
                         g4link_idx (ff, bb, ff, dd, aa, tbb, aa, tdd, r) +
                         g4link_idx (mm, bb, ff, dd, aa, tbb, aa, tdd, r) +
                         g4link_idx (ff, bb, mm, dd, aa, tbb, aa, tdd, r));
         }
         return xx;
      }
      else
      {
         dd = min (bb, dd);             /* now have dd as only one != aa */
         if (dd == bb)
         {
            tdd = tbb;
         }
         xx = 0.25 * (kinship (nghd + mm, nghd + dd) +
                      kinship (nghd + ff, nghd + dd) +
                      g4link_idx (mm, ff, mm, dd, aa, aa, aa, tdd, r) +
                      g4link_idx (mm, ff, ff, dd, aa, aa, aa, tdd, r));
         return xx;
      }
   }
   else
   {
      if (aa == bb)
      {
         if (isNullNode (nghd[aa].ma) && isNullNode (nghd[aa].pa))
            /* if (isNullNode (&nghd[mm]) && isNullNode (&nghd[ff])) */
         {
            return 0.5 * kinship (nghd + cc, nghd + dd);
         }
         else
         {
            xx = (0.5 * (kinship (nghd + cc, nghd + dd) +
                         g4link_idx (mm, ff, cc, dd, aa, aa, tcc, tdd, r)));
            return xx;
         }
      }
      else
      {
         if (isNullNode (nghd[aa].ma) && isNullNode (nghd[aa].pa))
            /*  if (isNullNode (&nghd[mm]) && isNullNode (&nghd[ff])) */
         {
            return 0.0;
         }
         else
         {
            xx = (0.5 * (g4link_idx (mm, bb, cc, dd, aa, tbb, tcc, tdd, r) +
                         g4link_idx (ff, bb, cc, dd, aa, tbb, tcc, tdd, r)));
            return xx;
         }
      }
   }
}
