/*  functions are:
 *          findParents()
 *          checkParents()
 */
#include "pedchk.h"

/*----------------------------------------------------------------------------*/
/* findParents():  looks for parents' records                                 */

void  findParents (void)
{
   PED_PTR pP = pPedDat;
   KEYS_PP ppK = ppKeyTab;
   KEYS_PP ppMa, ppPa;
   KEYS  find;
   KEYS_PTRS pf = &find;
   int   i;

   for (i = 0; i < nInd; ++i)           /* for each entry in key table  */
   {
      pP = pPedDat + (*ppK)->ix - 1;

      /*                                 * gets ma's name from ped data */
      strncpy (find.nam, pP->namMa, MAXNAMLEN + 1);
      if (!unknown (find.nam))
      {                                 /* finds ma's name in key table */
         ppMa = bsearch (&pf, ppKeyTab,
                         nInd, sizeof (KEYS_PTRS), comparNam);
         if (ppMa)
            (*ppK)->ixMa = (*ppMa)->ix; /* & stores ix in key table     */
      }

      /*                                 * gets pa's name from ped data */
      strncpy (find.nam, pP->namPa, MAXNAMLEN + 1);
      if (!unknown (find.nam))
      {                                 /* finds pa's name in key table */
         ppPa = bsearch (&pf, ppKeyTab,
                         nInd, sizeof (KEYS_PTRS), comparNam);
         if (ppPa)
            (*ppK)->ixPa = (*ppPa)->ix; /* & stores ix in key table     */
      }
      ++ppK;
   }
}

int   unknown (char *name)
{
   if (strncmp (name, UNKNOWNNAME, MAXNAMLEN) == 0)
      return 1;
   else
      return 0;
}

/*----------------------------------------------------------------------------*/
/* checkParents():  checks for missing parents or not both specified          */

int   checkParents (void)
{
   int   ixPed;
   char  ma[MAXNAMLEN + 1];
   char  pa[MAXNAMLEN + 1];
   char  namOneNamed[MAXNAMLEN + 1];
   char  namBothMiss[MAXNAMLEN + 1];
   char  namOneMiss[MAXNAMLEN + 1];

   int   i;
   int   nOneNamed, nBothMiss, nOneMiss;

   KEYS_PP ppK = ppKeyTab;

   nOneNamed = 0, nBothMiss = 0, nOneMiss = 0;

   for (i = 0; i < nInd; ++i)           /* for each entry in key table        */
   {
      ixPed = (*ppK)->ix - 1;
      /* gets ma's & pa's nam from ped data */
      strncpy (ma, (pPedDat + ixPed)->namMa, MAXNAMLEN + 1);
      strncpy (pa, (pPedDat + ixPed)->namPa, MAXNAMLEN + 1);

      if (unknown (ma) && unknown (pa)) /* checks for founder              */
         (*ppK)->founder = TRUE;

      else if (unknown (ma) || unknown (pa))  /* checks for only one parent */
      {
         ++nOneNamed;
         if (nOneNamed == 1)
            strncpy (namOneNamed, (pPedDat + ixPed)->nam, MAXNAMLEN + 1);
      }
      else if (!(*ppK)->ixMa && !(*ppK)->ixPa)  /* checks for parents         */
      {                                 /*         not in pedigree            */
         ++nBothMiss;
         if (nBothMiss == 1)
            strncpy (namBothMiss, (pPedDat + ixPed)->nam, MAXNAMLEN + 1);
      }
      else if (!(*ppK)->ixMa || !(*ppK)->ixPa)  /* checks for one parent      */
      {                                 /*         not in pedigree            */
         ++nOneMiss;
         if (nOneMiss == 1)
            strncpy (namOneMiss, (pPedDat + ixPed)->nam, MAXNAMLEN + 1);
      }
      ++ppK;
   }

   if (nOneNamed)
   {
      if (nOneNamed == 1)
         give_err (ERC_1p1, "checkParents", namOneNamed);
      else
         give_err (ERC_1pr, "checkParents", nOneNamed, namOneNamed);
      return !OK;
   }
   if (nBothMiss)
   {
      if (nBothMiss == 1)
         give_err (ERC_0p1, "checkParents", namBothMiss);
      else
         give_err (ERC_0pr, "checkParents", nBothMiss, namBothMiss);
      return !OK;
   }
   if (nOneMiss)
   {
      if (nOneMiss == 1)
         give_err (ERC_0p1, "checkParents", namOneMiss);
      else
         give_err (ERC_0pr, "checkParents", nOneMiss, namOneMiss);
      return !OK;
   }
   return OK;
}
