
/*************************************************************************

 function to generate random normal deviate x - N(0,1). 

  Source: Numerical Recipe in FORTRAN (p.202-203)

**************************************************************************/

#include "rans.h"

double normran (void)
{
   double v1, v2, r, fac;
   static int set = 0;              /* Note this must be STATIC */
   static double gset;              /* Note this must be STATIC */

   if (set)
   {
      set = 0;
      return gset;                      /* return the value computed 
                                         * in last call              */
   }
   else
   {
      do
      {
         v1 = 2.0 * rans () - 1.0;
         v2 = 2.0 * rans () - 1.0;
         r = v1 * v1 + v2 * v2;
      }
      while (r > 1.0 || r == 0.0);
      fac = sqrt (-2.0 * log (r) / r);
      gset = v1 * fac;                  /* save for next call */
      set = 1;
      return v2 * fac;
   }
}

/*************************************************************************

  program name: normal.c

  function to generate random normal deviate x - N(a, b^2)

  Date: 2/13/90 (SWG)
  Corrected, April 1992; (EAT)
  Improved, February 1994;  (CW)

**************************************************************************/

double normal (double a, double b)
{
   return a + b * normran ();
}
