/*****************************************************************************
  routine name: sampling.c

  Take out the MCMC sampling portion of lm_auto program, 
  to separate from scoring of results, and accommodate burn-in etc.

  EAT  Sept 2002
******************************************************************************/

#include "lsample.h"
#include "msample.h"

void  sampling (int t, int mm, int nloc, int nsw, int ngenes,
                int *perm_loc, int *perm_sw,
                int ***y, double **prob, int qt)
/* mm=0 if L-sample samples all loci, mm=1 if markers only  */
{
   int   k, l, i, jj, sx, c;

   if (rans () < LSamplProb)
   {                                    /* ***** Do L-sample *****  */
      if (!((t + 1) % ChkIt))
      {
         printf (" Starting L-sampler iteration %d\n", t + 1);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         permute (perm_loc, nloc + 1);  /* permute loci for L-sampler */
         for (k = 0; k <= nloc; ++k)
         {                              /* do l-sampling for ea. locus */
            jj = perm_loc[k];
            if (jj >= mm)               /* if mm=1, don't do jj=0  */
               (void) do_lsmpl (nloc, jj, prob, y, ngenes, qt);
         }
      }
      else
      {                                 /* ****   step   **** */

         jj = intran (mm, nloc);        /* choose a random locus
                                           * do l-sampling for this locus */
         (void) do_lsmpl (nloc, jj, prob, y, ngenes, qt);

      }
   }
   else
   {                                    /* ***** Do M-sample *****  */
      if (!((t + 1) % ChkIt))
      {
         printf (" Starting M-sampler iteration %d\n", t + 1);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         permute (perm_sw, nsw);        /* permute switches for M-sampler */
         for (k = 0; k < nsw; ++k)
         {                              /* for ea. switch, get meiosis */
            l = perm_sw[k] + 1;         /* and component */
            i = (meiosis[l].ind).index;
            sx = meiosis[l].sex;
            c = nghd[i].component - 1;
            /*                        * do m-sampling for the meiosis */
            do_msmpl (nloc, c, i, sx, prob, y, ngenes);
         }
      }
      else
      {                                 /* ****   step   **** */

         l = intran (0, nsw - 1) + 1;   /* choose a random switch */
         i = (meiosis[l].ind).index;
         sx = meiosis[l].sex;           /* get meiosis and component */
         c = nghd[i].component - 1;
         /*                           * do m-sampling for this meiosis */
         do_msmpl (nloc, c, i, sx, prob, y, ngenes);
      }
   }
}
